//*********************************************************************
// Skeleton of Windows application
//*********************************************************************

#ifndef STRICT
#define STRICT
#endif

#include <windows.h>
#include <stdio.h>
#include <stdarg.h>
#include "EatCPU.h"
#include "mainwnd.h"
#include "rhsreg.h"


//*********************************************************************
//
//*********************************************************************

static BOOL InitInstance(HINSTANCE, int);

static DWORD WINAPI CPUThreadProc(LPVOID Args);
static int EatSomeCPU(void);


//*********************************************************************
// Global variables
//*********************************************************************

HINSTANCE   hMainInst;
CRhsMainWnd MainWnd;
HANDLE      hCPUThread;

int Calibration = 0;

CRhsRegistry EatCPURegistry;


//*********************************************************************
// Entry point from Windows supervisor
//*********************************************************************

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{ MSG msg;
  HWND hWnd;

// If a previous instance is running re-activate it and exit

  if ((hWnd = FindWindow(String_IDS_MAINCLASSNAME, NULL)))
  { hWnd = GetLastActivePopup(hWnd);
    if (IsIconic(hWnd))
      ShowWindow(hWnd, SW_RESTORE);
    BringWindowToTop(hWnd);
    SetForegroundWindow(hWnd);
    return(0);
  }

// Initialise application

  if (!InitInstance(hInstance, nCmdShow))
    return(0);

// Enter message loop

  while (GetMessage(&msg, NULL, 0, 0))
  { TranslateMessage(&msg);
    DispatchMessage(&msg);
  }

  return(msg.wParam);
}


//*********************************************************************
// Instance initialisation routine
//*********************************************************************

static BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{

// Set the registry key

  EatCPURegistry.SetKey("Software\\Rhs\\EatCPU");

  Calibration = EatCPURegistry.GetInt("Calibration", 0);

// Create the window

  hMainInst = hInstance;

  if (!MainWnd.Open(hInstance, String_IDS_TITLE, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, "EatCPUMenu", LoadIcon(hMainInst, "BasicAppIcon")))
  { winprintf("Cannot create main window");
    return(FALSE);
  }

// Now create the thread

  DWORD d;

  hCPUThread = CreateThread(NULL, 0, CPUThreadProc, NULL, CREATE_SUSPENDED, &d);

  if (!hCPUThread)
  { winprintf("Error creating thread:\r\n%s", GetLastErrorMessage());
    return(FALSE);
  }

// Check if we have been calibrated

  if (Calibration == 0)
    winprintf("Wait until NT is quiet then choose File/Calibrate to set the reference point for performance testing.");

// Return indicating success

  return(TRUE);
}


//*********************************************************************
//*********************************************************************

void Calibrate(void)
{
  Calibration = EatSomeCPU();
  EatCPURegistry.WriteInt("Calibration", Calibration);
  MainWnd.OnPaint(NULL);
  winprintf("No. iterations = %i", Calibration);
}


void StartCounting(void)
{
  ResumeThread(hCPUThread);
}


void StopCounting(void)
{
  SuspendThread(hCPUThread);
}


//*********************************************************************
//*********************************************************************

static DWORD WINAPI CPUThreadProc(LPVOID Args)
{ int iterations;

// Start eating CPU

  for (;;)
  {

// Eat some CPU

    iterations = EatSomeCPU();

// If we are calibrated pass info to main window

    if (Calibration > 0)
      MainWnd.SetCounters((100*iterations)/Calibration, iterations);
  }

// Should never reach here

  return(0);
}


static int EatSomeCPU(void)
{ int iterations, i, j;
  DWORD endtime;

// Loop for two seconds

    endtime = GetTickCount() + 2000;
    iterations = 0;

    for (i = 0; endtime > GetTickCount(); i++)
    {

// Anything will do to use a bit of cpu

      j = 1234;
      j = j*j;
      j = j/j;

// Increment the iterations every 16 cycles

      if (i % 256 == 0)
        iterations++;
    }

// Return the number of iterations

  return(iterations);
}


//*********************************************************************
//*********************************************************************

const char* GetLastErrorMessage(void)
{ DWORD d;
  static char errmsg[512];

  d = GetLastError();

  strcpy(errmsg, "<unknown error>");
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, d, 0, errmsg, 511, NULL);
  return(errmsg);
}


//*********************************************************************
// FUNCTION: winprintf
//
// ARGUMENTS: Format - printf format string
//            ...    - vararg list
//
// RETURNS: Always returns IDOK
//
// DESCRIPTION:
//   Windows version of printf, the output is displayed in a message
//   box.
//   To use this function you must #include <stdio.h> and <stdarg.h>.
//*********************************************************************

int winprintf(char *Format, ...)
{ char s[256];
  va_list ap;

  va_start(ap, Format);
  vsprintf(s, Format, ap);
  va_end(ap);

  return(MessageBox(GetFocus(), s, String_IDS_TITLE, MB_OK | MB_ICONEXCLAMATION));
}


