//**********************************************************************
//**********************************************************************

#ifndef STRICT
#define STRICT
#endif

#include <windows.h>
#include <stdio.h>
#include "EatCPU.h"
#include "EatCPUDlg.h"
#include "rhsreg.h"
#include "mainwnd.h"


//**********************************************************************
//**********************************************************************

long CALLBACK CRhsMainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK CRhsMainWndAboutProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);


//**********************************************************************
//**********************************************************************

CRhsMainWnd::CRhsMainWnd(void)
{
  m_hInstance = NULL;
  m_hWnd = NULL;
  m_Registry.SetKey("Software\\Rhs\\EatCPU");
}


//**********************************************************************
//**********************************************************************

CRhsMainWnd::~CRhsMainWnd(void)
{
  if (m_hWnd)
    DestroyWindow(m_hWnd);
}


//**********************************************************************
//**********************************************************************

BOOL CRhsMainWnd::Open(HINSTANCE hInstance, char* Title, int Left, int Top, int Width, int Depth, char* Menu, HICON hIcon, HBRUSH hBrush)
{ WNDCLASS wc;
  RECT r;

// If the window has already been opened then just resize it

  if (m_hWnd)
  { MoveWindow(m_hWnd, Left, Top, Width, Depth, TRUE);
    return(TRUE);
  }

// Save the instance handle

  m_hInstance = hInstance;

// Register the parent window class

  wc.style         = 0;
  wc.lpfnWndProc   = CRhsMainWndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 4;
  wc.hInstance     = m_hInstance;
  wc.hIcon         = hIcon;
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = hBrush ? hBrush : (HBRUSH) (COLOR_WINDOW + 1);
  wc.lpszMenuName  = Menu;
  wc.lpszClassName = "CRhsMainWndClass";

  RegisterClass(&wc);

// Check if default size is required

  r.left   = Left  == CW_USEDEFAULT ? m_Registry.GetInt("Left",  CW_USEDEFAULT) + 24 : Left;
  r.top    = Top   == CW_USEDEFAULT ? m_Registry.GetInt("Top",   CW_USEDEFAULT) + 24 : Top;
  r.right  = Width == CW_USEDEFAULT ? m_Registry.GetInt("Width", CW_USEDEFAULT) : Width;
  r.bottom = Depth == CW_USEDEFAULT ? m_Registry.GetInt("Depth", CW_USEDEFAULT) : Depth;

// Create the window

  m_hWnd = CreateWindow("CRhsMainWndClass",
                        Title,
                        WS_OVERLAPPEDWINDOW,
                        r.left, r.top, r.right, r.bottom,
                        NULL,
                        NULL,
                        m_hInstance,
                        NULL
                       );

  if (!m_hWnd)
    return(FALSE);

  FitOnScreen(TRUE, TRUE);

// Save our object in the parent windows extra data

  SetWindowLong(m_hWnd, 0, (LONG) this);

// Show the window

  ShowWindow(m_hWnd, SW_SHOWNORMAL);
  UpdateWindow(m_hWnd);

// Return indicating success

  return(TRUE);
}


//**********************************************************************
//**********************************************************************

BOOL CRhsMainWnd::FitOnScreen(BOOL Origin, BOOL Size)
{ int width, depth;
  RECT r;

// Check the window exists

  if (!m_hWnd)
    return(FALSE);

// Get some dimensions

  GetWindowRect(m_hWnd, &r);
  width = GetSystemMetrics(SM_CXSCREEN);
  depth = GetSystemMetrics(SM_CYSCREEN);

// If required move the window so its top left corner is on screen

  if (Origin)
  { r.left = r.left >= 0 ? r.left : 0;
    r.top  = r.top >= 0 ? r.top : 0;
    r.left = r.left < width ? r.left : 0;
    r.top  = r.top < depth ? r.top : 0;
    SetWindowPos(m_hWnd, NULL, r.left, r.top, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
  }

// If required resize and move the window so it's wholly on screen

  if (Size)
  { r.right  -= r.left;
    r.bottom -= r.top;
    r.right  = r.right <= width ? r.right : width;
    r.bottom = r.bottom <= depth ? r.bottom : depth;
    r.left = r.left + r.right < width ? r.left : 0;
    r.top  = r.top + r.bottom < depth ? r.top  : 0;
    SetWindowPos(m_hWnd, NULL, r.left, r.top, r.right, r.bottom, SWP_NOZORDER);
  }

// Return indicating success

  return(TRUE);
}


//**********************************************************************
//**********************************************************************

void CRhsMainWnd::SetCounters(int Percent, int Count)
{ HDC hdc;
  char s[256];

  hdc = GetDC(m_hWnd);

  SelectObject(hdc, GetStockObject(ANSI_VAR_FONT));

  sprintf(s, "Speed = %i%%      ", Percent);
  TextOut(hdc, 0, 0, s, strlen(s));
  sprintf(s, "Iterations = %i      ", Count);
  TextOut(hdc, 0, 24, s, strlen(s));

  ReleaseDC(m_hWnd, hdc);
}


//**********************************************************************
//**********************************************************************

long CRhsMainWnd::OnCommand(WORD wNotifyCode, WORD wID, HWND hwndCtl)
{

// Process the message

  switch (wID)
  {

// Calibrate

    case IDM_FILE_CALIBRATE:
      Calibrate();
      return(0);

// Calibrate

    case IDM_FILE_GO:
      StartCounting();
      return(0);

// Stop counting

    case IDM_FILE_STOP:
      StopCounting();
      return(0);

// Exit

    case IDM_FILE_EXIT:
      DestroyWindow(m_hWnd);
      return(0);

// Traditional about box

    case IDM_FILE_ABOUT:
      DialogBox(m_hInstance, "AboutBox", m_hWnd, (DLGPROC) CRhsMainWndAboutProc);
      return(0);
  }

  return(0);
}


//**********************************************************************
//**********************************************************************

long CRhsMainWnd::OnSize(UINT fwSizeType)
{ RECT r;

  if (fwSizeType == SIZE_RESTORED)
  { GetWindowRect(m_hWnd, &r);
    m_Registry.WriteInt("Left",  r.left);
    m_Registry.WriteInt("Top",   r.top);
    m_Registry.WriteInt("Width", r.right - r.left);
    m_Registry.WriteInt("Depth", r.bottom - r.top);
  }

  return(0);
}


//**********************************************************************
//**********************************************************************

long CRhsMainWnd::OnMove(void)
{ RECT r;

  if (!IsIconic(m_hWnd) && !IsZoomed(m_hWnd))
  { GetWindowRect(m_hWnd, &r);
    m_Registry.WriteInt("Left", r.left);
    m_Registry.WriteInt("Top", r.top);
  }

  return(0);
}


//**********************************************************************
//**********************************************************************

long CRhsMainWnd::OnPaint(HDC hDC)
{ HDC hdc;
  char s[256];

  hdc = hDC ? hDC : GetDC(m_hWnd);

  SelectObject(hdc, GetStockObject(ANSI_VAR_FONT));

  sprintf(s, "Calibration = %i      ", m_Registry.GetInt("Calibration", 0));
  TextOut(hdc, 0, 48, s, strlen(s));

  if (!hDC)
    ReleaseDC(m_hWnd, hdc);

  return(0);
}


//**********************************************************************
//**********************************************************************

long CRhsMainWnd::OnDestroy(void)
{
  PostQuitMessage(0);
  return(0);
}


//**********************************************************************
//**********************************************************************

long CALLBACK CRhsMainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{ CRhsMainWnd* mainwnd;
  HDC hdc;
  PAINTSTRUCT ps;

// Get the object

  mainwnd = (CRhsMainWnd*) GetWindowLong(hWnd, 0);
  if (!mainwnd)
    return(DefWindowProc(hWnd, message, wParam, lParam));

// Other messages

  switch (message)
  {

// Message from menu or control

    case WM_COMMAND:
      return(mainwnd->OnCommand(HIWORD(wParam), LOWORD(wParam), (HWND) lParam));

// Window moved

    case WM_MOVE:
      return(mainwnd->OnMove());

// Window resized

    case WM_SIZE:
      return(mainwnd->OnSize((UINT) wParam));

// Window needs repainting

    case WM_PAINT:
      hdc = BeginPaint(hWnd, &ps);
      mainwnd->OnPaint(hdc);
      EndPaint(hWnd, &ps);
      break;

// Window destroyed

    case WM_DESTROY:
      return(mainwnd->OnDestroy());
  }

// Pass unhandled messages to Windows

  return(DefWindowProc(hWnd, message, wParam, lParam));
}


/***********************************************************************
* Message handler for About dialog box                                 *
***********************************************************************/

BOOL CALLBACK CRhsMainWndAboutProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{

  switch (message)
  { case WM_INITDIALOG:
      return(TRUE);

    case WM_COMMAND:
      switch (LOWORD(wParam))
      { case IDOK:
          EndDialog(hDlg, TRUE);
          return(TRUE);

        case IDD_ABOUT_INFO:
          DialogBox(NULL, "RegisterBox", hDlg, (DLGPROC) CRhsMainWndAboutProc);
          EndDialog(hDlg, TRUE);
          return(TRUE);
      }
      break;
  }

  return(FALSE);
}


