//**********************************************************************
//**********************************************************************

#ifndef STRICT
#define STRICT
#endif

#include <windows.h>
#include <string.h>
#include "rhsreg.h"


//**********************************************************************
//**********************************************************************

CRhsRegistry::CRhsRegistry(void)
{
  strcpy(m_Key, "");
}


CRhsRegistry::CRhsRegistry(char* Key)
{
  strcpy(m_Key, "");

  if (Key)
    if (strlen(Key) <= LEN_REGISTRYPATH)
      strcpy(m_Key, Key);
}


CRhsRegistry::~CRhsRegistry(void)
{
}


//**********************************************************************
//**********************************************************************

BOOL CRhsRegistry::SetKey(char* Key)
{
  if (!Key)
    return(FALSE);

  if (strlen(Key) > LEN_REGISTRYPATH)
    return(FALSE);

  strcpy(m_Key, Key);

  return(TRUE);
}


BOOL CRhsRegistry::GetKey(char* Key, int Len)
{
  if ((int) strlen(m_Key) > Len)
    return(FALSE);

  strcpy(Key, m_Key);

  return(TRUE);
}


//**********************************************************************
//**********************************************************************

BOOL CRhsRegistry::WriteString(char* Key, char* String)
{ HKEY h;
  LONG l;
  DWORD d;

// Check parameters

  if (!Key || !String)
    return(FALSE);

// Write to the registry

  l = RegCreateKeyEx(HKEY_CURRENT_USER, m_Key, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &h, &d);
  if (l != ERROR_SUCCESS)
    return(FALSE);

  l = RegSetValueEx(h, Key, 0, REG_SZ, (CONST BYTE*) String, strlen(String) + 1);
  RegCloseKey(h);

  if (l != ERROR_SUCCESS)
    return(FALSE);

// Return indicating success

  return(TRUE);
}


//**********************************************************************
//**********************************************************************

BOOL CRhsRegistry::WriteInt(char* Key, int Value)
{ HKEY h;
  LONG l;
  DWORD value, d;

// Check parameters

  if (!Key)
    return(FALSE);

// Write to the registry

  l = RegCreateKeyEx(HKEY_CURRENT_USER, m_Key, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &h, &d);
  if (l != ERROR_SUCCESS)
    return(FALSE);

  value = (DWORD) Value;
  l = RegSetValueEx(h, Key, 0, REG_DWORD, (CONST BYTE*) &value, sizeof(DWORD));
  RegCloseKey(h);

  if (l != ERROR_SUCCESS)
    return(FALSE);

// Return indicating success

  return(TRUE);
}


//**********************************************************************
//**********************************************************************

char* CRhsRegistry::GetString(char* Key, char* Default, int Len)
{ DWORD len;
  HKEY h;
  LONG l;
  DWORD regtype, d;

// An internal 4 string buffer is used to store read strings

  static id = 0;
  char   string[4][LEN_REGISTRYPATH+1];

// Check parameters

  id = id % 4;

  if (!Key)
    return(NULL);

// Read from the registry

  len = Len;

  l = RegCreateKeyEx(HKEY_CURRENT_USER, m_Key, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &h, &d);
  if (l != ERROR_SUCCESS)
    return(FALSE);

  l = RegQueryValueEx(h, Key, NULL, &regtype, (LPBYTE) string[id], (DWORD*) &len);
  RegCloseKey(h);

// If necessary use the default

  if (l != ERROR_SUCCESS)
  { if (Default)
    { strncpy(string[id], Default, LEN_REGISTRYPATH);
      string[id][LEN_REGISTRYPATH+1] = '\0';
    }
    else
    { strcpy(string[id], "");
    }
  }

// Return length of string

  return(string[id++]);
}


//**********************************************************************
//**********************************************************************

int CRhsRegistry::GetInt(char* Key, int Default)
{ HKEY h;
  LONG l;
  DWORD regtype, value, len, d;

// Check parameters

  if (!Key)
    return(Default);

// Read from the registry

  l = RegCreateKeyEx(HKEY_CURRENT_USER, m_Key, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &h, &d);
  if (l != ERROR_SUCCESS)
    return(FALSE);

  len = sizeof(DWORD);
  l = RegQueryValueEx(h, Key, NULL, &regtype, (LPBYTE) &value, (DWORD*) &len);
  RegCloseKey(h);

  if (l != ERROR_SUCCESS)
    return(Default);

  if (regtype != REG_DWORD)
    return(Default);

  return((int) value);
}


